@@ -118,14 +118,19 @@ class Test_document_fromstring(unittest.TestCase):
 
     def test_basic(self):
         parser = DummyParser(doc=DummyElementTree(root='dummy root'))
-        elem = self.call_it('dummy input', parser=parser)
+        elem = self.call_it(b'dummy input', parser=parser)
         self.assertEqual(elem, 'dummy root')
-        self.assertEqual(parser.parse_args, ('dummy input',))
+        self.assertEqual(parser.parse_args, (b'dummy input',))
         self.assertEqual(parser.parse_kwargs, {'useChardet': True})
 
+    def test_guess_charset_not_used_for_unicode(self):
+        parser = DummyParser()
+        elem = self.call_it(b''.decode('ascii'), parser=parser)
+        self.assertEqual(parser.parse_kwargs, {})
+
     def test_guess_charset_arg_gets_passed_to_parser(self):
         parser = DummyParser()
-        elem = self.call_it('', guess_charset='gc_arg', parser=parser)
+        elem = self.call_it(b'', guess_charset='gc_arg', parser=parser)
         self.assertEqual(parser.parse_kwargs, {'useChardet': 'gc_arg'})
 
     def test_raises_type_error_on_nonstring_input(self):
@@ -145,14 +150,20 @@ class Test_fragments_fromstring(unittest.TestCase):
 
     def test_basic(self):
         parser = DummyParser(fragments='fragments')
-        fragments = self.call_it('dummy input', parser=parser)
+        fragments = self.call_it(b'dummy input', parser=parser)
         self.assertEqual(fragments, 'fragments')
+        self.assertEqual(parser.parseFragment_kwargs, {'useChardet': False})
 
     def test_guess_charset_arg_gets_passed_to_parser(self):
         parser = DummyParser()
-        elem = self.call_it('', guess_charset='gc_arg', parser=parser)
+        elem = self.call_it(b'', guess_charset='gc_arg', parser=parser)
         self.assertEqual(parser.parseFragment_kwargs, {'useChardet': 'gc_arg'})
 
+    def test_guess_charset_not_used_for_unicode(self):
+        parser = DummyParser()
+        elem = self.call_it(b''.decode('ascii'), parser=parser)
+        self.assertEqual(parser.parseFragment_kwargs, {})
+
     def test_raises_type_error_on_nonstring_input(self):
         not_a_string = None
         self.assertRaises(TypeError, self.call_it, not_a_string)
