@@ -14,6 +14,7 @@ if this_dir not in sys.path:
 from common_imports import etree, HelperTestCase, fileInTestDir
 from common_imports import doctest, make_doctest
 
+
 class ETreeISOSchematronTestCase(HelperTestCase):
     def test_schematron(self):
         tree_valid = self.parse('<AAA><BBB/><CCC/></AAA>')
@@ -37,6 +38,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
     </pattern>
 </schema>
 ''')
+
         schema = isoschematron.Schematron(schema)
         self.assertTrue(schema.validate(tree_valid))
         self.assertTrue(not schema.validate(tree_invalid))
@@ -55,7 +57,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
 ''')
         schema = isoschematron.Schematron(schema)
         self.assertTrue(schema)
-        
+
     def test_schematron_invalid_schema_empty(self):
         schema = self.parse('''\
 <schema xmlns="http://purl.oclc.org/dsdl/schematron" />
@@ -291,7 +293,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
         schematron = isoschematron.Schematron(schema)
         self.assertTrue(schematron.validator_xslt is None)
 
-        schematron = isoschematron.Schematron(schema, store_schematron=True) 
+        schematron = isoschematron.Schematron(schema, store_schematron=True)
         self.assertTrue(isinstance(schematron.schematron, etree._ElementTree),
                      'expected schematron schema to be stored')
 
@@ -309,10 +311,10 @@ class ETreeISOSchematronTestCase(HelperTestCase):
         schematron = isoschematron.Schematron(schema)
         self.assertTrue(schematron.validator_xslt is None)
 
-        schematron = isoschematron.Schematron(schema, store_xslt=True) 
+        schematron = isoschematron.Schematron(schema, store_xslt=True)
         self.assertTrue(isinstance(schematron.validator_xslt, etree._ElementTree),
                      'expected validator xslt to be stored')
-       
+
     def test_schematron_abstract(self):
         schema = self.parse('''\
 <sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
@@ -322,7 +324,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
 
   <!-- of course, these only really make sense when combined with a schema that
        ensures datatype xs:dateTime -->
-       
+
   <sch:pattern abstract="true" id="abstract.dateTime.tz_utc">
     <sch:rule context="$datetime">
       <sch:let name="tz" value="concat(substring-after(substring-after(./text(), 'T'), '+'), substring-after(substring-after(./text(), 'T'), '-'))"/>
@@ -369,7 +371,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
 </root>
 '''),
             ]
-                       
+
         schematron = isoschematron.Schematron(schema)
         for tree_valid in valid_trees:
             self.assertTrue(schematron(tree_valid), schematron.error_log)
@@ -426,7 +428,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
 
   <!-- of course, these only really make sense when combined with a schema that
        ensures datatype xs:dateTime -->
-  
+
   <sch:pattern abstract="true" id="abstract.dateTime.tz_utc">
     <sch:rule context="$datetime">
       <sch:let name="tz" value="concat(substring-after(substring-after(./text(), 'T'), '+'), substring-after(substring-after(./text(), 'T'), '-'))"/>
@@ -526,7 +528,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
             len(schematron.error_log), expected,
             'expected %s errors: %s (%s errors)' %
             (expected, schematron.error_log, len(schematron.error_log)))
-                                      
+
     def test_schematron_phases_kwarg(self):
         schema = self.parse('''\
 <sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
@@ -551,7 +553,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
 
   <!-- of course, these only really make sense when combined with a schema that
        ensures datatype xs:dateTime -->
-  
+
   <sch:pattern abstract="true" id="abstract.dateTime.tz_utc">
     <sch:rule context="$datetime">
       <sch:let name="tz" value="concat(substring-after(substring-after(./text(), 'T'), '+'), substring-after(substring-after(./text(), 'T'), '-'))"/>
@@ -647,7 +649,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
         self.assertEqual(
             len(schematron.error_log), expected, 'expected %s errors: %s (%s errors)' %
             (expected, schematron.error_log, len(schematron.error_log)))
-                                      
+
     def test_schematron_xmlschema_embedded(self):
         schema = self.parse('''\
 <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
@@ -785,13 +787,13 @@ class ETreeISOSchematronTestCase(HelperTestCase):
                     namespaces={'sch': "http://purl.oclc.org/dsdl/schematron"})
                     or [None])[0]
                 return schematron
-                              
+
             def _include(self, schematron, **kwargs):
                 raise RuntimeError('inclusion unsupported')
-            
+
             def _expand(self, schematron, **kwargs):
                 raise RuntimeError('expansion unsupported')
-            
+
             def _validation_errors(self, validationReport):
                 valid = etree.XPath(
                     'count(//svrl:successful-report[@flag="critical"])=1',
@@ -831,7 +833,7 @@ class ETreeISOSchematronTestCase(HelperTestCase):
         # check if overridden _expand is run
         self.assertRaises(RuntimeError, MySchematron, schema, store_report=True,
                           include=False)
-        
+
         schema = MySchematron(schema, store_report=True, include=False,
                               expand=False)
         self.assertTrue(schema.validate(tree_valid))
@@ -839,10 +841,32 @@ class ETreeISOSchematronTestCase(HelperTestCase):
 
     #TODO: test xslt parameters for inclusion, expand & compile steps (?)
 
+    def test_schematron_fail_on_report(self):
+        tree_valid = self.parse('<AAA><BBB/><CCC/></AAA>')
+        tree_invalid = self.parse('<AAA><BBB/><CCC/><DDD/></AAA>')
+        schema = self.parse('''\
+<schema xmlns="http://purl.oclc.org/dsdl/schematron" >
+    <pattern id="OpenModel">
+        <title>Simple Report</title>
+        <rule context="AAA">
+            <report test="DDD"> DDD element must not be present</report>
+        </rule>
+    </pattern>
+</schema>
+''')
+        schema_report = isoschematron.Schematron(
+            schema, error_finder=isoschematron.Schematron.ASSERTS_AND_REPORTS)
+        schema_no_report = isoschematron.Schematron(schema)
+        self.assertTrue(schema_report.validate(tree_valid))
+        self.assertTrue(not schema_report.validate(tree_invalid))
+        self.assertTrue(schema_no_report.validate(tree_valid))
+        self.assertTrue(schema_no_report.validate(tree_invalid))
+
 
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeISOSchematronTestCase)])
+    suite.addTests(doctest.DocTestSuite(isoschematron))
     suite.addTests(
         [make_doctest('../../../doc/validation.txt')])
     return suite
